<?php
/**
 * Static class that will handle all ajax calls for the list
 *
 * @author MoreConvert
 * @package Smart Wishlist For More Convert
 * @version 1.3.3
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}// Exit if accessed directly

if ( ! class_exists( 'WLFMC_Ajax_Handler' ) ) {
	/**
	 * Woocommerce Smart Wishlist Ajax Handler
	 */
	class WLFMC_Ajax_Handler {
		/**
		 * Performs all required add_actions to handle forms
		 *
		 * @return void
		 * @version 1.3.3
		 */
		public static function init() {

			// add to wishlist.
			add_action( 'wp_ajax_wlfmc_add_to_wishlist', array( 'WLFMC_Ajax_Handler', 'add_to_wishlist' ) );
			add_action( 'wp_ajax_nopriv_wlfmc_add_to_wishlist', array( 'WLFMC_Ajax_Handler', 'add_to_wishlist' ) );

			// add to cart.
			add_action( 'wp_ajax_wlfmc_add_to_cart', array( 'WLFMC_Ajax_Handler', 'add_to_cart' ) );
			add_action( 'wp_ajax_nopriv_wlfmc_add_to_cart', array( 'WLFMC_Ajax_Handler', 'add_to_cart' ) );

			// remove from wishlist.
			add_action( 'wp_ajax_wlfmc_remove_from_wishlist', array( 'WLFMC_Ajax_Handler', 'remove_from_wishlist' ) );
			add_action(
				'wp_ajax_nopriv_wlfmc_remove_from_wishlist',
				array(
					'WLFMC_Ajax_Handler',
					'remove_from_wishlist',
				)
			);

			// remove from wishlist (button).
			add_action( 'wp_ajax_wlfmc_delete_item', array( 'WLFMC_Ajax_Handler', 'delete_item' ) );
			add_action( 'wp_ajax_nopriv_wlfmc_delete_item', array( 'WLFMC_Ajax_Handler', 'delete_item' ) );

			// update item quantity.
			add_action( 'wp_ajax_wlfmc_update_item_quantity', array( 'WLFMC_Ajax_Handler', 'update_item_quantity' ) );
			add_action(
				'wp_ajax_nopriv_wlfmc_update_item_quantity',
				array(
					'WLFMC_Ajax_Handler',
					'update_item_quantity',
				)
			);

			// load fragments.
			add_action( 'wp_ajax_wlfmc_load_fragments', array( 'WLFMC_Ajax_Handler', 'load_fragments' ) );
			add_action( 'wp_ajax_nopriv_wlfmc_load_fragments', array( 'WLFMC_Ajax_Handler', 'load_fragments' ) );

			// load analytics just for logged-in user.
			add_action( 'wp_ajax_wlfmc_load_analytics', array( 'WLFMC_Ajax_Handler', 'load_analytics' ) );

			// wp_loaded ajax_mode.
			add_action( 'wp_loaded', array( 'WLFMC_Ajax_Handler', 'wp_loaded_action' ), 0 );

		}

		/**
		 * Manage all wp_loaded ajax mode actions.
		 *
		 * @return false|void
		 * @since 1.3.3
		 */
		public static function wp_loaded_action() {
			if ( is_null( filter_input( INPUT_POST, 'action' ) ) ) {
				return false;
			}

			$action = ! empty( $_POST['action'] ) ? sanitize_key( $_POST['action'] ) : '';// phpcs:ignore WordPress.Security.NonceVerification

			if ( ! in_array(
				$action,
				array(
					'wlfmc_wp_loaded_add_to_wishlist',
					'wlfmc_wp_loaded_load_analytics',
					'wlfmc_wp_loaded_remove_from_wishlist',
					'wlfmc_wp_loaded_update_item_quantity',
					'wlfmc_wp_loaded_delete_item',
					'wlfmc_wp_loaded_load_fragments',
				),
				true
			) ) {
				return false;
			}
			remove_action( 'init', 'woocommerce_add_to_cart_action' );
			remove_action( 'wp_loaded', 'WC_Form_Handler::add_to_cart_action', 20 );

			$action = str_replace( 'wlfmc_wp_loaded_', '', $action );
			self::$action();

		}

		/**
		 * Add to cart item
		 *
		 * @return void
		 * @throws Exception If add to cart fails.
		 * @since 1.4.2
		 */
		public static function add_to_cart() {

			$item_id     = isset( $_POST['lid'] ) ? absint( wp_unslash( $_POST['lid'] ) ) : '';// phpcs:ignore WordPress.Security.NonceVerification
			$wishlist_id = isset( $_POST['wid'] ) ? absint( wp_unslash( $_POST['wid'] ) ) : '';// phpcs:ignore WordPress.Security.NonceVerification

			if ( ! $item_id || ! $wishlist_id ) {
				return;
			}

			$wishlist = wlfmc_get_wishlist( $wishlist_id );

			if ( ! $wishlist ) {
				return;
			}

			$item = $wishlist->get_item( $item_id );

			if ( ! $item ) {
				return;
			}
			$options                  = new MCT_Options( 'wlfmc_options' );
			$remove_after_add_to_cart = 'added-to-cart' === $options->get_option( 'remove_from_wishlist', 'none' );
			$product_id               = $item->get_product_id();
			$quantity                 = $item->get_quantity();
			$meta                     = $item->get_product_meta( 'view' );
			$cart_item                = $item->get_cart_item();
			$product                  = $item->get_product();
			$attributes               = array();
			if ( isset( $meta['attributes'] ) && ! empty( $meta['attributes'] ) ) {
				foreach ( $meta['attributes'] as $key => $value ) {
					if ( '' !== $value ) {
						$attributes[ $key ] = $value;
					}
				}
			}
			$variation_id = 0;
			if ( $product && 'variation' === $product->get_type() ) {
				$variation_id = $product_id;
				$product_id   = $product->get_parent_id();
			}
			$passed_validation = apply_filters( 'wlfmc_woocommerce_add_to_cart_validation', true, $product, $meta, $item, $cart_item );
			if ( $passed_validation ) {

				$cart_item_key = WC()->cart->add_to_cart( $product_id, $quantity, $variation_id, $attributes, $cart_item );

				if ( false !== $cart_item_key ) {
					wc_add_to_cart_message( array( $product_id => $quantity ), true );
					if ( $remove_after_add_to_cart ) {
						$item->delete();
						do_action( 'wlfmc_product_added_to_cart', $wishlist->get_id(), $cart_item_key, $product_id, $quantity );
					}
				}
				WC_AJAX::get_refreshed_fragments();

			} else {

				$permalink    = apply_filters( 'woocommerce_cart_item_permalink', $product->is_visible() ? $product->get_permalink( $cart_item ) : '', $cart_item, '' );
				$redirect_url = apply_filters( 'woocommerce_cart_redirect_after_error', $permalink, $product_id );
				// If there was an error adding to the cart, redirect to the product page to show any errors.
				$data = array(
					'error'       => true,
					'product_url' => $redirect_url,
				);
				wp_send_json( $data );
			}
		}

		/**
		 * Add to wishlist from ajax call
		 *
		 * @return void
		 * @version 1.3.3
		 */
		public static function add_to_wishlist() {
			$result = false;
			try {
				$result  = WLFMC()->add();
				$return  = 'true';
				$message = '';

			} catch ( WLFMC_Exception $e ) {
				$return  = $e->getTextualCode();
				$message = apply_filters( 'wlfmc_error_adding_to_wishlist_message', $e->getMessage() );
			} catch ( Exception $e ) {
				$return  = 'error';
				$message = apply_filters( 'wlfmc_error_adding_to_wishlist_message', $e->getMessage() );
			}
			$product_id    = isset( $_REQUEST['add_to_wishlist'] ) ? intval( $_REQUEST['add_to_wishlist'] ) : false; // phpcs:ignore WordPress.Security
			$fragments     = isset( $_REQUEST['fragments'] ) ? json_decode( wp_unslash( $_REQUEST['fragments'] ), true ) : false;// phpcs:ignore WordPress.Security
			$found_item    = false;
			$found_in_list = false;
			if ( 'exists' === $return ) {
				$found_in_list = wlfmc_get_wishlist( false );
				$found_item    = $found_in_list ? $found_in_list->get_product( $product_id ) : false;
			}

			$wishlist_url = is_user_logged_in() ? apply_filters( 'wlfmc_user_logged_in_wishlist_url', wc_get_account_endpoint_url( get_option( 'woocommerce_myaccount_wlfmc_wishlist_endpoint', 'wlfmc-wishlist' ) ) ) : WLFMC()->get_last_operation_url();

			wp_send_json(
				apply_filters(
					'wlfmc_ajax_add_return_params',
					array(
						'prod_id'      => $product_id,
						'result'       => $return,
						'message'      => $message,
						'fragments'    => self::refresh_fragments( $fragments ),
						'wishlist_url' => $wishlist_url,
						'wishlist_id'  => $result ? $result['wishlist_id'] : ( $found_in_list ? $found_in_list->get_id() : false ),
						'item_id'      => $result ? $result['item_id'] : ( $found_item ? $found_item->get_id() : false ),
					)
				)
			);
		}

		/**
		 *  Add new hook for work analytics after product added to wishlist
		 *
		 * @return void
		 */
		public static function load_analytics() {
			$product_id  = isset( $_POST['product_id'] ) ? intval( $_POST['product_id'] ) : false; // phpcs:ignore WordPress.Security.NonceVerification
			$wishlist_id = isset( $_POST['wishlist_id'] ) ? intval( $_POST['wishlist_id'] ) : false; // phpcs:ignore WordPress.Security.NonceVerification
			if ( ! $wishlist_id || ! $product_id ) {
				die();
			}

			do_action( 'wlfmc_analytics_added_to_wishlist', $product_id, $wishlist_id, get_current_user_id() );

			// stops ajax call from further execution (no return value expected on answer body).
			wp_send_json_success();
		}

		/**
		 * Remove from wishlist from ajax call
		 *
		 * @return void
		 * @version 1.3.0
		 */
		public static function remove_from_wishlist() {
			$fragments = isset( $_REQUEST['fragments'] ) ? wp_unslash( $_REQUEST['fragments'] ) : false;// phpcs:ignore WordPress.Security
			$return    = false;
			$result    = false;
			try {
				$result  = WLFMC()->remove();
				$return  = 'true';
				$message = '';
			} catch ( Exception $e ) {
				$message = $e->getMessage();
			}

			wp_send_json(
				array(
					'result'    => $return,
					'message'   => $message,
					'count'     => $result ? $result['count'] : false,
					'fragments' => self::refresh_fragments( $fragments ),
				)
			);
		}

		/**
		 * Remove item from a wishlist
		 * Differs from remove from wishlist, since this accepts item id instead of product id
		 *
		 * @return void
		 * @version 1.3.0
		 */
		public static function delete_item() {
			$item_id   = isset( $_POST['item_id'] ) ? intval( $_POST['item_id'] ) : false; // phpcs:ignore WordPress.Security.NonceVerification
			$fragments = isset( $_REQUEST['fragments'] ) ? wp_unslash( $_REQUEST['fragments'] ) : false;// phpcs:ignore WordPress.Security
			$return    = array(
				'result' => false,
			);

			if ( $item_id ) {
				$item = WLFMC_Wishlist_Factory::get_wishlist_item( $item_id );

				if ( $item ) {
					$item->delete();
					$message = '';

					$return = array(
						'result'    => 'true',
						'message'   => $message,
						'fragments' => self::refresh_fragments( $fragments ),
					);
				}
			}

			wp_send_json( $return );
		}

		/**
		 * Update quantity of an item in wishlist
		 *
		 * @return void
		 */
		public static function update_item_quantity() {
			$wishlist_token = isset( $_POST['wishlist_token'] ) ? sanitize_text_field( wp_unslash( $_POST['wishlist_token'] ) ) : false; // phpcs:ignore WordPress.Security.NonceVerification
			$product_id     = isset( $_POST['product_id'] ) ? intval( $_POST['product_id'] ) : false; // phpcs:ignore WordPress.Security.NonceVerification
			$quantity       = isset( $_POST['quantity'] ) ? intval( $_POST['quantity'] ) : 1; // phpcs:ignore WordPress.Security.NonceVerification

			if ( ! $wishlist_token || ! $product_id ) {
				die();
			}

			$wishlist = wlfmc_get_wishlist( $wishlist_token );

			if ( ! $wishlist || ! $wishlist->current_user_can( 'update_quantity' ) ) {
				die();
			}

			$item = $wishlist->get_product( $product_id );

			if ( ! $item ) {
				die();
			}

			$item->set_quantity( $quantity );
			$item->save();

			// stops ajax call from further execution (no return value expected on answer body).
			die();
		}

		/**
		 * Generated fragments to replace in the page
		 *
		 * @return void
		 */
		public static function load_fragments() {
			$fragments = isset( $_POST['fragments'] ) ? wp_unslash( $_POST['fragments'] ) : false;// phpcs:ignore WordPress.Security

			if ( defined( 'ICL_SITEPRESS_VERSION' ) ) { // wpml current  language.
				global $sitepress;
				$lang = $sitepress->get_current_language();
			} elseif ( function_exists( 'pll_current_language' ) ) { // polylang current language.
				$lang = pll_current_language();
			} else { // cannot determine current language.
				$lang = null;
			}

			wp_send_json(
				array(
					'fragments' => self::refresh_fragments( $fragments ),
					'products'  => self::get_current_items(),
					'lang'      => $lang,
				)
			);
		}

		/**
		 * Generate fragments for the templates that needs to be refreshed after ajax
		 *
		 * @param array|null $fragments Array of fragments to refresh.
		 *
		 * @return array Array of templates to be replaced on the page
		 * @version 1.3.3
		 */
		public static function refresh_fragments( $fragments ): array {
			$result = array();
			if ( ! empty( $fragments ) ) {
				foreach ( $fragments as $id => $options ) {

					$id      = sanitize_text_field( $id );
					$options = WLFMC_Frontend()->decode_fragment_options( $options );
					$item    = $options['item'] ?? false;
					if ( ! $item ) {
						continue;
					}

					switch ( $item ) {
						case 'wishlist_counter':
						case 'wishlist':
							$result[ $id ] = WLFMC_Shortcode::$item( $options );
							break;
						default:
							$result[ $id ] = apply_filters( 'wlfmc_fragment_output', '', $id, $options );
							break;
					}
				}
			}

			return $result;
		}

		/**
		 * Get current items in current user wishlist
		 *
		 * @return array
		 */
		public static function get_current_items(): array {
			$wishlist_data = array();

			$wishlist = WLFMC_Wishlist_Factory::get_current_wishlist();

			if ( ! empty( $wishlist ) ) {
				$items = $wishlist->get_items();
				if ( ! empty( $items ) ) {
					foreach ( $items as $item ) {
						$wishlist_data[] = array(
							'product_id'  => $item->get_product_id(),
							'wishlist_id' => $wishlist->get_id(),
							'item_id'     => $item->get_id(),
						);
					}
				}
			}
			return $wishlist_data;
		}
	}
}
WLFMC_Ajax_Handler::init();
