<?php
/**
 * WLFMC wishlist integration with oceanwp theme
 *
 * @author MoreConvert
 * @package Smart Wishlist For More Convert
 * @version 1.3.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

add_action( 'init', 'wlfmc_oceanwp_integrate' );

/**
 * Integration with Oceanwp theme
 *
 * @return void
 */
function wlfmc_oceanwp_integrate() {

	if ( class_exists( 'OCEANWP_Theme_Class' ) ) {

		add_filter( 'wlfmc_button_positions', 'wlfmc_oceanwp_fix_single_position' );
		add_filter( 'wlfmc_loop_positions', 'wlfmc_oceanwp_fix_loop_position' );

		// fix inline loop buttons.
		$options       = new MCT_Options( 'wlfmc_options' );
		$loop_position = $options->get_option( 'loop_position', 'after_add_to_cart' );

		if ( in_array( $loop_position, array( 'before_add_to_cart', 'after_add_to_cart' ), true ) ) {

			add_action( 'ocean_before_archive_product_add_to_cart_inner', 'wlfmc_oceanwp_inline_btn_loop_start', 9 );
			add_action( 'ocean_after_archive_product_add_to_cart_inner', 'wlfmc_oceanwp_inline_btn_loop_end', 11 );

		}
	}

}

/**
 * Start add wrapper for button position in loop
 *
 * @return void
 */
function wlfmc_oceanwp_inline_btn_loop_start() {
	echo '<div class="wlfmc-flex-buttons">';
}

/**
 * End add wrapper for button position in loop
 *
 * @return void
 */
function wlfmc_oceanwp_inline_btn_loop_end() {
	echo '</div>';
}

/**
 * Fix single position
 *
 * @param array $positions all single positions.
 *
 * @return array
 */
function wlfmc_oceanwp_fix_single_position( array $positions ): array {

	$positions['image_top_left']     = array(
		array(
			'hook'     => 'woocommerce_product_thumbnails',
			'priority' => '21',
		),
		array(
			'hook'     => 'ocean_woo_quick_view_product_image',
			'priority' => '50',
		),
	);
	$positions['image_top_right']    = array(
		array(
			'hook'     => 'woocommerce_product_thumbnails',
			'priority' => '21',
		),
		array(
			'hook'     => 'ocean_woo_quick_view_product_image',
			'priority' => '50',
		),
	);
	$positions['image_bottom_left']  = array(
		array(
			'hook'     => 'woocommerce_product_thumbnails',
			'priority' => '21',
		),
		array(
			'hook'     => 'ocean_woo_quick_view_product_image',
			'priority' => '50',
		),
	);
	$positions['image_bottom_right'] = array(
		array(
			'hook'     => 'woocommerce_product_thumbnails',
			'priority' => '21',
		),
		array(
			'hook'     => 'ocean_woo_quick_view_product_image',
			'priority' => '50',
		),
	);
	$positions['summary']            = array(
		array(
			'hook'     => 'ocean_woo_quick_view_product_content',
			'priority' => '100',
		),
		array(
			'hook'     => 'woocommerce_single_product_summary',
			'priority' => '100',
		),
	);

	return $positions;
}

/**
 * Fix loop position
 *
 * @param array $positions all loop positions.
 *
 * @return array
 */
function wlfmc_oceanwp_fix_loop_position( array $positions ): array {

	$positions['before_add_to_cart']['hook']     = 'ocean_before_archive_product_add_to_cart_inner';
	$positions['before_add_to_cart']['priority'] = '10';
	$positions['after_add_to_cart']['hook']      = 'ocean_after_archive_product_add_to_cart_inner';
	$positions['after_add_to_cart']['priority']  = '10';
	$positions['image_top_left']['hook']         = 'ocean_before_archive_product_image';
	$positions['image_top_right']['hook']        = 'ocean_before_archive_product_image';

	return $positions;
}
