<?php
/**
 * WLFMC wishlist integration with shoptimizer theme
 *
 * @author MoreConvert
 * @package Smart Wishlist For More Convert
 * @since 1.4.3
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

add_action( 'init', 'wlfmc_shoptimizer_integrate' );

add_filter( 'wlfmc_wishlist_button_settings', 'wlfmc_shoptimizer_single_button_positions' );

/**
 * Integration with Shoptimizer theme
 *
 * @return void
 */
function wlfmc_shoptimizer_integrate() {

	if ( class_exists( 'CommerceGurus_Commercekit' ) ) {

		add_action(
			'wlfmc_table_product_before_add_to_cart',
			function() {
				remove_filter( 'woocommerce_loop_add_to_cart_link', 'commercegurus_as_loop_add_to_cart_link', 99 );
			}
		);

		add_action(
			'wlfmc_table_product_after_add_to_cart',
			function() {
				add_filter( 'woocommerce_loop_add_to_cart_link', 'commercegurus_as_loop_add_to_cart_link', 99, 2 );
			}
		);

		$shoptimizer_layout_woocommerce_card_display = '';
		$shoptimizer_layout_woocommerce_card_display = function_exists( 'shoptimizer_get_option' ) ? shoptimizer_get_option( 'shoptimizer_layout_woocommerce_card_display' ) : 'slide';
		if ( 'slide' !== $shoptimizer_layout_woocommerce_card_display ) {
			add_filter( 'wlfmc_loop_positions', 'wlfmc_shoptimizer_fix_loop_position' );

		}

		add_filter( 'wlfmc_button_positions', 'wlfmc_shoptimizer_fix_single_position' );
		remove_action( 'woocommerce_single_product_summary', 'commercekit_single_product_wishlist', 38 );
		remove_action( 'woocommerce_before_shop_loop_item_title', 'commercekit_after_shop_loop_item_wishlist', 15 );
	}

}

/**
 * Fix single position
 *
 * @param array $positions all single positions.
 *
 * @return array
 */
function wlfmc_shoptimizer_fix_single_position( array $positions ): array {

	$positions['image_top_left']  = array(
		array(
			'hook'     => 'commercekit_before_gallery',
			'priority' => '10',
		),
	);
	$positions['image_top_right'] = array(
		array(
			'hook'     => 'commercekit_before_gallery',
			'priority' => '10',
		),
	);
	return $positions;
}

/**
 * Fix loop position
 *
 * @param array $positions all loop positions.
 *
 * @return array
 */
function wlfmc_shoptimizer_fix_loop_position( array $positions ): array {
	$positions['after_add_to_cart'] = array(
		'hook'     => 'woocommerce_after_shop_loop_item',
		'priority' => 7,
	);
	return $positions;
}

/**
 * Remove single position not compatible with shoptimizer
 *
 * @param array $settings wishlist single position settings.
 *
 * @return array
 */
function wlfmc_shoptimizer_single_button_positions( array $settings ): array {

	if ( class_exists( 'CommerceGurus_Commercekit' ) ) {
		if ( isset( $settings['wishlist_button_position']['options']['before_add_to_cart_button'] ) ) {
			unset( $settings['wishlist_button_position']['options']['before_add_to_cart_button'] );
		}
		if ( isset( $settings['wishlist_button_position']['options']['after_add_to_cart_button'] ) ) {
			unset( $settings['wishlist_button_position']['options']['after_add_to_cart_button'] );
		}
		if ( isset( $settings['wishlist_button_position']['options']['image_bottom_left'] ) ) {
			unset( $settings['wishlist_button_position']['options']['image_bottom_left'] );
		}
		if ( isset( $settings['wishlist_button_position']['options']['image_bottom_right'] ) ) {
			unset( $settings['wishlist_button_position']['options']['image_bottom_right'] );
		}
	}

	return $settings;
}
