<?php
/**
 * WLFMC wishlist integration with woocommerce bookings plugin
 *
 * @name WooCommerce Bookings
 * @version 1.15.63
 * @slug woocommerce-bookings
 * @url https://woocommerce.com/products/woocommerce-bookings/

 * @author MoreConvert
 * @package Smart Wishlist For More Convert
 * @since 1.4.3
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}
add_action( 'wlfmc_adding_to_wishlist', 'wlfmc_fix_woocommerce_bookings_add_to_wishlist' );

/**
 * Fix add to wishlist woocommerce bookings
 *
 * @return void
 */
function wlfmc_fix_woocommerce_bookings_add_to_wishlist() {
	if ( class_exists( 'WC_Bookings_Addons' ) ) {
		add_filter( 'wlfmc_add_cart_item_data', 'wlmfc_woocommerce_bookings_cart_item_data', 10, 2 );
		add_filter( 'woocommerce_booking_form_get_posted_data', 'wlfmc_disable_woocommerce_bookings_cart_item_data', 10 );
		if ( class_exists( 'WC_Product_Addons' ) ) {
			// Woocommerce addons integrate.
			$wc_bookings_addons = new WC_Bookings_Addons();
			add_filter( 'wlfmc_woocommerce_addons_add_cart_item_data', array( $wc_bookings_addons, 'add_cart_item_data_adjust_booking_cost' ), 20 );
			add_filter( 'wlfmc_woocommerce_addons_add_cart_item_data', array( $GLOBALS['Product_Addon_Cart'], 'add_cart_item_data' ), 10, 2 );

		}
	}
}

/**
 *  Empty posted data before add to wishlist for avoid adding to in-cart status.
 *
 * @return array
 */
function wlfmc_disable_woocommerce_bookings_cart_item_data() {
	return array();
}

/**
 * Fix woocommerce booking cart item data.
 *
 * @param array $cart_item_data cart item data.
 * @param int   $product_id product_id.
 *
 * @return array
 */
function wlmfc_woocommerce_bookings_cart_item_data( $cart_item_data, $product_id ) {
	remove_filter( 'woocommerce_booking_form_get_posted_data', 'wlfmc_disable_woocommerce_bookings_cart_item_data', 10 );
	$product = wc_get_product( $product_id );
	if ( ! is_wc_booking_product( $product ) ) {
		return $cart_item_data;
	}

	if ( ! key_exists( 'booking', $cart_item_data ) ) {
		$cart_item_data['booking'] = wc_bookings_get_posted_data( $_POST, $product ); // phpcs:ignore WordPress.Security.NonceVerification
	}
	$cart_item_data['booking']['_cost'] = WC_Bookings_Cost_Calculation::calculate_booking_cost( $cart_item_data['booking'], $product );

	if ( $cart_item_data['booking']['_cost'] instanceof WP_Error ) {
		$cart_item_data['booking']['_cost'] = 0;
	}
	add_filter( 'woocommerce_booking_form_get_posted_data', 'wlfmc_disable_woocommerce_bookings_cart_item_data', 10, 3 );

	return (array) apply_filters( 'wlfmc_woocommerce_addons_add_cart_item_data', $cart_item_data, $product_id );
}
